-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
--Mandatory controller parameters
M.type = "auxiliary"
M.relevantDevice = nil

local abs = math.abs

local barrelNodes = nil
local bulletTimer = nil
local numberOfRounds = nil
local roundIndex = nil
local gunState = nil
local desiredHandleValue = nil
local handleSmoother = newTemporalSmoothing(40, 60)
local fireRateTime = nil
local firingSound = nil

local bulletBeams = {}
local bulletNodes = {}
local activeBullets = {}

local function updateGFX(dt)
  if gunState ~= "idle" or electrics.values.mg_fire >= 1 then --only go in here if we are STILL firing or want to fire
    bulletTimer = bulletTimer + dt

    if gunState == "idle" and electrics.values.mg_fire >= 1 and roundIndex <= numberOfRounds then --if we can fire, want to fire and still have rounds left...
      electrics.values["bllt_"..roundIndex] = 1 --roundIndex starts at 1, and determines which bullet is to be fired. it is added to "bllt_", so if roundIndex is 1 then the thruster with "bllt_1" as the control is activated
      electrics.values.recoil = 1 --activates the recoil thruster
      electrics.values.muzf = 1	--show the muzzleflash. This is done with glowmaps with "muzf" as their input.
      desiredHandleValue = 1
      sounds.playSoundOnceAtNode(firingSound, barrelNodes[2], 5)	--Play the firing sound
      obj:addParticleByNodesRelative(barrelNodes[2], barrelNodes[1], -10, 40, 0, 1)
      gui.message("Ammo: " .. numberOfRounds - roundIndex) --updates the ammo counter
      gunState = "firing"

    elseif gunState == "firing" then
      if bulletTimer >= 0.03 then	--if the bullet timer goes over 0.03 seconds,
        electrics.values["bllt_"..roundIndex] = nil --turn off the bullet thruster
        electrics.values.recoil = nil --turn of the recoil thruster
        electrics.values.muzf = 0 --hide muzzleflash
        desiredHandleValue = 0
        table.insert(activeBullets, {beamCid = bulletBeams[roundIndex], nodeCid = bulletNodes[roundIndex]})
        gunState = "fired"
      end

    elseif gunState == "fired" then
      if bulletTimer >= fireRateTime then --after x seconds since firing (this adjusts rate of fire),
        bulletTimer = 0	--reset bullet timer for next shot
        roundIndex = roundIndex + 1 --cycle "roundIndex". So if it was 1, it is now 2. When the code runs again, it will fire shell 2, then shell 3 and so on
        gunState = "idle"
      end
    end
  end

  local finishedBullets = {}
  for k,v in ipairs(activeBullets) do
    local bulletVelocity = abs(obj:getNodeVelocityVector(v.nodeCid):length())
    if bulletVelocity <= 150 then
      obj:breakBeam(v.beamCid)
      table.insert(finishedBullets, k)
    end
  end

  for k,v in ipairs(finishedBullets) do
    table.remove(activeBullets, v)
  end

  electrics.values.mg_barrel = handleSmoother:getUncapped(desiredHandleValue, dt)
end

local function init(jbeamData)
  electrics.values.mg_barrel = 0
  electrics.values.mg_fire = 0

  bulletTimer = 0
  roundIndex = 1
  numberOfRounds = jbeamData.numberOfRounds or 0
  firingSound = jbeamData.firingSound or "CrashTestSound"
  fireRateTime = 1 / (jbeamData.rateOfFire or 1)
  gunState = "idle"
  desiredHandleValue = 0

  handleSmoother:reset()

  barrelNodes = {}
  bulletNodes = {}
  for k, node in pairs (v.data.nodes) do
    if node.name == "brrl_l0" then barrelNodes[1] = k  end
    if node.name == "brrl_r1" then barrelNodes[2] = k  end

    if node.bullet then bulletNodes[node.bullet] = k  end
  end

  activeBullets = {}
  bulletBeams = {}
  for k, beam in pairs(v.data.beams) do
    if beam.tracer then bulletBeams[beam.tracer] = k  end
  end
end

-- public interface
M.init = init
M.updateGFX = updateGFX
return M



